open Pprintdata

(**************** Pretty printer library *****************)

(* The datatypes doc and sdoc and mode, as well as the helper functions
 * smartcons, conslist, empty, text, nest, br, breakWith and group,
 * are all given in pprintdata.mli.  Their implementations are 
 * pre-compiled in pprintdata.cmi and pprintdata.cmo.

 * Also included in pprintdata.mli are reference implementations of
 * fits, format, pretty, sdocToString and stmtToDoc.  To use these functions
 * to develop solutions to the homework out of order, replace the 
 * definitions below e.g.
 *     let rec fits w imdList = Pprintdata.fits w imdList
 * (which, in better style, would just be "let fits = Pprintdata.fits")
 * Your submitted solutions should obviously not use these, and will be 
 * tested without the pre-compiled reference. *)

let rec sdocToString doc =
  match doc with
  | SNil        -> raise Unimplemented
  | SText(s,d)  -> raise Unimplemented
  | SLine(i,d)  -> raise Unimplemented


(* fits determines whether a specified document fits within the 
 * remaining space of a given line width.
 * The imdList parameter is a list of triples (indent level, mode, document) 
 * w is the space left on the current line *)
let rec fits w imdList = 
  raise Unimplemented

(* format converts a doc into an sdoc.  This essentially translates
 * from the more convenient representation, with groups and brs,
 * into the lower-level representation with just strings and spaces.
 * imdList is the same as above
 * w is the width of the document
 * k is the number of characters already consumed on this line *)
let rec format w k imdList = 
  raise Unimplemented

(* pretty actually does the pretty printing.  It converts a document
 * into a sdoc, then converts the sdoc into a string. *)
let pretty w doc = 
  raise Unimplemented

(* just toss a newline on to the end *)
let pretty_print w doc =
  print_endline (pretty w doc)

(*************** pretty-printer clients ******************)

open Ast

let rec expToDoc exp =
  match exp with
    Int i -> text (string_of_int i)
  | Var v -> text v
  | Plus(e1, e2) -> 
      group [nest 2 [text "(" ; br ; expToDoc e1 ;
                     br ; text "+" ; 
                     br ; expToDoc e2 ; br ; text ")"] ]
  | Times(e1, e2) -> 
      group [nest 2 [text "(" ; br ; expToDoc e1 ;
                     br ; text "*" ; 
                     br ; expToDoc e2 ; br ; text ")"] ]

let rec stmtToDoc stmt = 
  raise Unimplemented

type xmlAst = Element of 
  string                      (* name of element *)
    * (string * string) list  (* list of attributes, as (name,value) pairs *)
    * xmlAst list             (* list of children elements *)

let xmlToDoc xml = 
  raise Unimplemented

(******************** pretty-printer test *************)
let get_prog_f filename =
  let ch = open_in filename in
  let res = Parse.program Lex.lexer (Lexing.from_channel ch) in
  close_in ch;
  res
let get_prog_s str = Parse.program Lex.lexer (Lexing.from_string str)

let test_prog filename =
  raise Unimplemented

(********************* Other ***********************)

(* a possibly useful helper function that makes a string n characters wide
 * with the number n in the middle.  *)
let linewidth n = 
  let strn = string_of_int(n) in
  let charsstrn = String.length strn in
  let n = n-2-charsstrn in
  let (left, right) = if (n mod 2 = 0) 
                      then (n/2, n/2)
                      else ((n+1)/2,(n-1)/2) in
  "<" ^ (String.make left '-') ^ strn ^ (String.make right '-') ^ ">\n"

